"""
(c) 2020 АО 'НИИ "МАСШТАБ"'
"""

import unittest

from veilrpc.tests.networking import INITIAL_CONFIG, OVS_PARAMS, IFACES_CONFIGS
from veilrpc.states.networking import split_net_config, process_net_config, get_writable_iface_config, \
    gen_valid_ovs_params, get_iface_config_as_dict


class TestVeilrpcNetworkingFunctions(unittest.TestCase):

    def test_process_net_config(self):
        # pylint: disable=cell-var-from-loop
        raw_content = INITIAL_CONFIG
        splitted_conf, _ = split_net_config(raw_content)
        processed_conf = process_net_config(splitted_conf)
        self.assertIsInstance(processed_conf, list)
        for entry in processed_conf:
            self.assertIsInstance(entry, dict)
            map(lambda x: self.assertTrue(x in entry.keys()), ['name', 'mode'])
            writable_config = get_writable_iface_config(entry)
            if entry['mode'] == 'static':
                map(lambda x: self.assertTrue(x in entry.keys()), ['address', 'netmask'])
            if entry['mode'] == 'loopback':
                self.assertListEqual(['auto lo', 'iface lo inet loopback'], writable_config)
            if entry['mode'] == 'manual':
                if entry['name'] == 'veil-blackhole':
                    self.assertListEqual(
                        ["auto veil-blackhole", "iface veil-blackhole inet manual",
                         "    pre-up ovs-ofctl add-flow veil-blackhole 'table=0,action=drop'"], writable_config)
                else:
                    self.assertListEqual(['auto {}'.format(entry['name']),
                                          'iface {} inet manual'.format(entry['name'])], writable_config)
            if entry['mode'] == 'dhcp':
                self.assertListEqual(['auto {}'.format(entry['name']),
                                      'allow-hotplug {}'.format(entry['name']),
                                      'iface {} inet dhcp'.format(entry['name'])], writable_config)
            self.assertIsInstance(writable_config, list)

    def test_gen_valid_ovs_params(self):
        for params in OVS_PARAMS:
            result_str = gen_valid_ovs_params(**params['input'])
            self.assertEqual(result_str, params['output'])

    def test_get_iface_config_as_dict(self):
        for entry in IFACES_CONFIGS:
            iface_dict = get_iface_config_as_dict(entry['input'])
            self.assertDictEqual(iface_dict, entry['output'])
            iface_to_save = get_writable_iface_config(iface_dict)
            self.assertListEqual(iface_to_save, entry['output_to_save'])
