"""
(c) 2020 АО 'НИИ "МАСШТАБ"'
"""

import logging

from .base_object import VeilApiObject, asyncable

LOG = logging.getLogger("bazalt")


class DataPool(VeilApiObject):
    base_url = "data-pools/"

    def __str__(self):
        if self.data.type in ["local", "nfs", "glusterfs", "zfs"]:
            return "directory datapool ({})".format(self.uuid)
        else:
            return "{} datapool ({})".format(self.data.type, self.uuid)

    @classmethod
    @asyncable
    def create(cls, verbose_name, pool_type, options, node=None, zfs_pool=None, lun=None,
               shared_storage=None, description="", context=None):
        LOG.info("Datapool {} {} create".format(pool_type, verbose_name))
        kwargs = {
            "type": pool_type,
            "options": options,
            "description": description,
            "verbose_name": verbose_name
        }
        if shared_storage:
            kwargs["shared_storage"] = shared_storage
        if node:
            kwargs["node"] = node
        if zfs_pool:
            kwargs["zfs_pool"] = zfs_pool
        if lun:
            kwargs["lun"] = lun
        r = cls.api_session.post(cls.prepare_url(cls.base_url, context), data=kwargs)
        r.raise_for_status_code()
        return r, cls

    @asyncable
    def delete(self, force=False, context=None):
        LOG.info("Datapool {} delete".format(self.name))
        r = self.api_session.post(self.prepare_url(self.get_self_url(self.uuid) + "remove/", context),
                                  data={'force': force})
        r.raise_for_status_code()
        return r, None

    def unregister(self, context=None):
        LOG.info("Datapool {} unregister".format(self.name))
        r = self.api_session.post(self.prepare_url(self.get_self_url(self.uuid) + "unregister/", context))
        if context and context.get("dry-run"):
            return r
        else:
            r.raise_for_status_code()

    @asyncable
    def clear(self, context=None):
        r = self.api_session.post(self.prepare_url(self.get_self_url(self.uuid) + 'clear/', context),
                                  data={})
        r.raise_for_status_code()
        return r, None

    def free(self, context=None):
        LOG.info("Datapool {} free space".format(self.name))
        r = self.api_session.post(self.prepare_url(self.get_self_url(self.uuid) + "free/", context))
        if context and context.get("dry-run"):
            return r
        else:
            r.raise_for_status_code()
            return r.data

    def discover_iso(self, context=None):
        LOG.info("Datapool {} discover_iso".format(self.name))
        r = self.api_session.post(self.prepare_url(self.get_self_url(self.uuid) + "discover-iso/", context))
        if context and context.get("dry-run"):
            return r
        else:
            r.raise_for_status_code()
            return r.data

    def discover_vdisks(self, context=None):
        LOG.info("Datapool {} discover_vdisks".format(self.name))
        r = self.api_session.post(self.prepare_url(self.get_self_url(self.uuid) + "discover-vdisks/", context))
        if context and context.get("dry-run"):
            return r
        else:
            r.raise_for_status_code()
            return r.data

    def discover_files(self, context=None):
        LOG.info("Datapool {} discover_files".format(self.name))
        r = self.api_session.post(self.prepare_url(self.get_self_url(self.uuid) + "discover-files/", context))
        if context and context.get("dry-run"):
            return r
        else:
            r.raise_for_status_code()
            return r.data

    @property
    def name(self):
        return self.data.verbose_name
